package kdub;

import game.*;
import java.util.Random;

public class MyStrategy implements Strategy {

  private static final int RESPONSES = 24;

  private Random rng;
  private int n;
  private int[] m, myactions;
  private int[][] best_response, responses, history, A;
  private double[] reward, regret, pi;

  public MyStrategy(Game game, long seed, String options) {
    rng = new Random(seed);
    n   = game.getNumPlayers();
    m   = new int[n];
    int product = 1;
    for(int i=0; i<n; ++i) {
      m[i]     = game.getNumActions();
      product *= m[i];
    }
    history = new int[n][];
    for(int i=0; i<n; ++i) {
      history[i] = new int[m[i]];
    }
    reward  = new double[RESPONSES];
    regret  = new double[RESPONSES];
    pi      = new double[RESPONSES];
    Utility u = game.getUtility();
    A         = new int[n][product];
    best_response = new int[n][];
    responses     = new int[n][RESPONSES];
    myactions     = new int[n];
    for(int i=0; i<n; ++i) {
      int idx = 0;
      for(int j=0; j<n; ++j) {
	idx = m[j]*idx + myactions[j];
      }
      int[] result = u.getUtility(myactions);
      for(int j=0; j<n; ++j) {
	A[j][idx] = result[j];
      }
      int k = 0;
      for(; k<n; ++k) {
	++myactions[k];
	if (myactions[k] == m[k]) {
	  myactions[k] = 0;
	} else {
	  break;
	}
      }
      if (k == n) {
	break;
      }
    }
    for(int i=0; i<n; ++i) {
      for(int j=0; j<n; ++j) {
	myactions[j] = 0;
      }
      product = 1;
      for(int j=0; j<n; ++j) {
	if (j != i) {
	  product *= m[j];
	}
      }
      best_response[i] = new int[product];
      for(;;) {
	int best = -1;
	int bval = 0;
	for(int j=0; j<m[i]; ++j) {
	  myactions[i] = j;
	  int idx = 0;
	  for(int ii=0; ii<n; ++ii) {
	    idx = m[ii]*idx + myactions[ii];
	  }
	  int cval = A[i][idx];
	  if (best == -1 || cval > bval) {
	    best = j;
	    bval = cval;
	  }
	}
	int idx = 0;
	for(int j=0; j<n; ++j) {
	  if (i != j) {
	    idx = m[j]*idx + myactions[j];
	  }
	}
	best_response[i][idx] = best;
	int k = 0;
	for(; k<n; ++k) {
	  if (k != i) {
	    ++myactions[k];
	    if (myactions[k] == m[k]) {
	      myactions[k] = 0;
	    } else {
	      break;
	    }
	  }
	}
	if (k == n) {
	  break;
	}
      }
    }
  }

  public int getAction() {
    double dart = rng.nextDouble();
    for(int i=0; i<RESPONSES; ++i) {
      dart -= pi[i];
      if (dart < 0) {
	return responses[0][i];
      }
    }
    return rng.nextInt(m[0]);
  }

  public void observeOutcome(int[] actions) {
    int me    = actions[0];
    double ev = 0;
    for(int i=0; i<RESPONSES; ++i) {
      actions[0] = responses[0][i];
      int idx = 0;
      for(int j=0; j<n; ++j) {
	idx = m[j]*idx + actions[j];
      }
      reward[i]  = A[0][idx];
      ev         = reward[i]*pi[i];
    }
    actions[0] = me;
    for(int i=0; i<RESPONSES; ++i) {
      regret[i] += reward[i] - ev;
    }
    for(int i=0; i<n; ++i) {
      ++history[i][actions[i]];
    }
    for(int i=0; i<n; ++i) {
      int best    = -1;
      double bval = -1e100;
      for(int j=0; j<m[i]; ++j) {
	for(int ii=0; ii<n; ++ii) {
	  myactions[ii] = 0;
	}
	myactions[i] = j;
	double cval = 0;
	for(;;) {
	  int idx = 0;
	  for(int ii=0; ii<n; ++ii) {
	    idx = m[ii]*idx + myactions[ii];
	  }
	  double payoff = A[i][idx];
	  for(int ii=0; ii<n; ++ii) {
	    if (ii != i) {
	      payoff *= history[ii][myactions[ii]];
	    }
	  }
	  cval += payoff;
	  int k = 0;
	  for(; k<n; ++k) {
	    if (i != k) {
	      ++myactions[k];
	      if (myactions[k] == m[k]) {
		myactions[k] = 0;
	      } else {
		break;
	      }
	    }
	  }
	  if (k == n) {
	    break;
	  }
	}
	if (best == -1 || cval > bval) {
	  bval = cval;
	  best = j;
	}
      }
      responses[i][0] = best;
    }
    for(int i=1; i<RESPONSES; ++i) {
      for(int j=0; j<n; ++j) {
	int idx = 0;
	for(int k=0; k<n; ++k) {
	  if (j != k) {
	    idx = m[k]*idx + responses[k][i-1];
	  }
	}
	responses[j][i] = best_response[j][idx];
      }
    }
    double sum = 0;
    for(int i=0; i<RESPONSES; ++i) {
      if (regret[i] > 0) {
	sum += regret[i];
      }
    }
    if (sum > 0) {
      for(int i=0; i<RESPONSES; ++i) {
	if (regret[i] > 0) {
	  pi[i] = regret[i]/sum;
	} else {
	  pi[i] = 0;
	}
      }
    } else {
      for(int i=0; i<RESPONSES; ++i) {
	pi[i] = 1./RESPONSES;
      }
    }
  }
}