package brownstrategy;

import game.Game;
import game.Strategy;

import java.util.Random;
import java.util.Stack;

import brownstrategy.agent.Agent;
import brownstrategy.agent.DeterministicAgent;
import brownstrategy.agent.NondeterministicAgent;

/**
 * A strategy that chooses a random starting position and delta on the first round, and then
 * moves according to delta each turn.
 * @author ddufresn
 *
 */

public class ConstantDelta extends NondeterministicAgent{

	int numActions;
	int lastAction;
	int delta;
	
	Stack<Integer> lastActionStack;

	/**
	 * Construct a constant delta strategy. The game is
	 * used to get the number of actions, and the
	 * seed is used to seed a pseudorandom number 
	 * generator, which then is queried for the action 
	 * to take.
	 * 
	 * @param g
	 * @param seed
	 */
	public ConstantDelta(Game g, long seed, String options) {
		super(g,seed,options);
		numActions = g.getNumActions();
		Random r = new Random(seed);
		lastAction = r.nextInt(numActions);
		delta=r.nextInt(numActions);
		lastActionStack = new Stack<Integer>();
	}



	@Override
	public Agent getCopy() {
		return new ConstantDelta(_g, _seed, _options);
	}

	@Override
	public Agent getCopy(Game g, long seed, String options) {
		return new ConstantDelta(g, seed, options);
	}

	@Override
	public double[] getActionDistribution() {
		if(_day == _cacheDay) {
			return _cacheDist;
		}
		_cacheDist = new double[12];
		_cacheDay = _day;
		lastAction = (lastAction+delta)%numActions;
		_cacheDist[lastAction] = 1;
		return _cacheDist;
	}

	@Override
	public void observeOutcome(int[] actions) {
		_day++;
	}



	@Override
	public void lookAhead(int[] actions) {
		lastActionStack.push(lastAction);
		observeOutcome(actions);
	}



	@Override
	public void lookBack() {
		lastAction = lastActionStack.pop();
		_day--;
	}
}
