package brownstrategy;

import game.Game;
import game.Strategy;

import java.util.Random;
import java.util.Stack;

import brownstrategy.agent.Agent;
import brownstrategy.agent.DeterministicAgent;
import brownstrategy.agent.NondeterministicAgent;

/**
 * 
 * This agent randomly mimics the move of one of our
 * opponents from the previous day.
 * 
 * @author jberg
 *
 */
public class MimicOpponent extends NondeterministicAgent {
	/**
	 * The number of actions available in the game.
	 */
	int numActions;
	
	/**
	 * The last actions that were observed
	 */
	int[] lastActions;

	/**
	 * A random number generator for the random actions.
	 */
	Random r;

	private Stack<int[]> lastActionsStack;
	
	public MimicOpponent(Game g, long seed, String options) {
		super(g,seed,options);
		this.numActions = g.getNumActions();
		this.lastActions = new int[3];
		this.r = new Random(seed);
		lastActionsStack = new Stack<int[]>();
	}

	@Override
	public Agent getCopy() {
		return new MimicOpponent(_g, _seed, _options);
	}

	@Override
	public Agent getCopy(Game g, long seed, String options) {
		return new MimicOpponent(g, seed, options);
	}

	@Override
	public double[] getActionDistribution() {
		if(_day == _cacheDay) {
			return _cacheDist;
		}
		_cacheDist = new double[12];
		_cacheDay = _day;
		int action;
		if(lastActions != null) {
			if(r.nextDouble() < .5) {
				action = lastActions[1];
			}
			else {
				action = lastActions[2];
			}
		}
		else {
			action = r.nextInt(numActions);
		}
		_cacheDist[action] = 1;
		return _cacheDist;
	}

	@Override
	public void observeOutcome(int[] actions) {
		lastActions = actions;
		_day++;
	}
	
	@Override
	public void lookAhead(int[] actions) {
		lastActionsStack.push(lastActions);
		observeOutcome(actions);
	}

	@Override
	public void lookBack() {
		lastActions = lastActionsStack.pop();
		_day--;
	}

}
