package brownstrategy.oracles;

import game.Game;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 *
 * This oracle makes its predictions based on each agents
 * moves individually.  For any given number of last moves
 * it checks for matching sequences in the agents history
 * and determines how it moved after that sequence of moves
 * previously seen.  This oracle only counts times that they
 * had the same utility (TOP,MID, or BOT).
 * 
 * @author jberg
 *
 */

public class PrevSeqIndividualWithUtilOracle extends AbstractPrevSequenceOracle {

	final static int BOT = 0;
	final static int MID = 1;
	final static int TOP = 2;

	public PrevSeqIndividualWithUtilOracle(Game g, long seed, String options) {
		super(g,seed,options);
	}

	public int[] getOppDist(int opp) {
		int[] arr = new int[12];
		ArrayList<Integer> moves;
		if(opp == 0) {
			moves = player1Moves;
		}
		else if (opp == 1) {
			moves = player2Moves;
		}
		else {
			moves = player3Moves;
		}
		if(_numLastMoves == 0 || moves.size() <= _numLastMoves) {
			for (Integer move : moves) {
				arr[move] = arr[move] + 1;
			}
		}
		else {
			int[] movesArr;
			if (opp == 2) {
				movesArr = new int[]{player2Moves.get(moves.size() - 1),player1Moves.get(moves.size() - 1),player3Moves.get(moves.size() - 1)};
			}
			else {
				movesArr = new int[]{player3Moves.get(moves.size() - 1),player1Moves.get(moves.size() - 1),player2Moves.get(moves.size() - 1)};
			}
			int util = getUtilRank(movesArr);
			List<Integer> prevMoves = moves.subList(moves.size() - _numLastMoves, moves.size());
			for(int i = 0; i < moves.size() - _numLastMoves; i++) {
				List<Integer> subMoves = moves.subList(i, i+_numLastMoves);
				if (arrListEquals(prevMoves, subMoves)) {
					int[] movesArr2;
					if (opp == 2) {
						movesArr2 = new int[]{player2Moves.get(i+_numLastMoves),player1Moves.get(i+_numLastMoves),player3Moves.get(i+_numLastMoves)};
					}
					else {
						movesArr2 = new int[]{player3Moves.get(i+_numLastMoves),player1Moves.get(i+_numLastMoves),player2Moves.get(i+_numLastMoves)};
					}
					int util2 = getUtilRank(movesArr2);
					if(util == util2) {
						arr[moves.get(i+_numLastMoves)] = arr[moves.get(i+_numLastMoves)] + 1;
					}
				}
			}
		}
		return arr;
	}

	public int getUtilRank(int[] moves) {
		int[] utils = new int[3];
		utils[0] = getUtility(moves);
		utils[1] = getUtility(moves[1],moves[0],moves[2]);
		utils[2] = getUtility(moves[2],moves[0],moves[1]);
		if(utils[0] >= utils[1]) {
			if(utils[0] >= utils[2]) {
				return TOP;
			}
			else {
				return MID;
			}
		}
		else if(utils[0] >= utils[2]) {
			return MID;
		}
		else {
			return BOT;
		}
	}

	public String toString() {
		return "PrevIndividualOpponentWithUtil(n = " + _numLastMoves + ")";
	}

	public PrevSeqIndividualWithUtilOracle getCopy() {
		return new PrevSeqIndividualWithUtilOracle(_g,_seed,_options);
	}
	
	public PrevSeqIndividualWithUtilOracle getCopy(Game g,long seed,String options) {
		return new PrevSeqIndividualWithUtilOracle(_g,_seed,_options);
	}

}