package brownstrategy.oracles;

import game.Game;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import util.Array;

/**
 *
 * This oracle makes its predictions based on all agents
 * moves.  For any given number of last moves
 * it checks for matching sequences in the agents history
 * and determines how it moved after that sequence of moves
 * previously seen.  This oracle also takes into consider
 * the rotational symmetry of the game
 * 
 * @author jberg
 *
 */

public class PrevSequenceAllRotSymOracle extends AbstractPrevSequenceOracle {

	public PrevSequenceAllRotSymOracle(Game g, long seed, String options) {
		super(g,seed,options);
	}

	public int[] getOppDist(int opp) {
		int[] arr = new int[12];
		ArrayList<Integer> opp1,opp2;
		ArrayList<Integer> playerBeingModeled;
		if (opp == 0) {
			opp1 = player2Moves;
			opp2 = player3Moves;
			playerBeingModeled = player1Moves;
		}
		else if (opp == 1) {
			opp1 = player1Moves;
			opp2 = player3Moves;
			playerBeingModeled = player2Moves;
		}
		else {
			opp1 = player1Moves;
			opp2 = player2Moves;
			playerBeingModeled = player3Moves;
		}
		if(_numLastMoves == 0 || playerBeingModeled.size() <= _numLastMoves) {
			for (Integer move : playerBeingModeled) {
				arr[move] = arr[move] + 1;
			}
		}
		else {
			List<Integer> prevMoves = opp1.subList(opp1.size() - _numLastMoves, opp1.size());
			List<Integer> ourPrevMoves = opp2.subList(opp2.size() - _numLastMoves, opp2.size());
			List<Integer> modelPrevMoves = playerBeingModeled.subList(opp2.size() - _numLastMoves, opp2.size());
			for(int i = 0; i < opp1.size() - _numLastMoves; i++) {
				List<Integer> subMoves = opp1.subList(i, i+_numLastMoves);
				List<Integer> ourSubMoves = opp2.subList(i, i+_numLastMoves);
				List<Integer> modelSubMoves = playerBeingModeled.subList(i, i+_numLastMoves);
				/*
				 * This checks to see if the distance between the two opponents
				 * is the same
				 */
				if (arrListEquals(getDiffList(modelPrevMoves, prevMoves), getDiffList(modelSubMoves, subMoves)) &&
					arrListEquals(getDiffList(modelPrevMoves, ourPrevMoves), getDiffList(modelSubMoves, ourSubMoves))) {
					int idx = (playerBeingModeled.get(playerBeingModeled.size()-1) + ((playerBeingModeled.get(i+_numLastMoves) - playerBeingModeled.get(i+_numLastMoves-1) + 12) % 12)) % 12;
					arr[idx] = arr[idx] + 1;
				}
			}
		}
		return arr;
	}
	
	public List<Integer> getDiffList(List<Integer> arrList1, List<Integer> arrList2) {
		ArrayList<Integer> diffList = new ArrayList<Integer>();
		for(int i = 0; i < arrList1.size(); i++) {
			diffList.add((arrList1.get(i) - arrList2.get(i) + 12) % 12);
		}
		return diffList;
	}
	
	public String toString() {
		return "PrevSequenceAllRotSymOracle(n = " + _numLastMoves + ")";
	}

	public PrevSequenceAllRotSymOracle getCopy() {
		return new PrevSequenceAllRotSymOracle(_g,_seed,_options);
	}
	
	@Override
	public PrevSequenceAllRotSymOracle getCopy(Game g, long seed, String options) {
		return new PrevSequenceAllRotSymOracle(g,seed,options);
	}
	
}