package brownstrategy.oracles;

import game.Game;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 *
 * This oracle makes its predictions based on each agents
 * moves individually.  For any given number of last moves
 * it checks for matching sequences in the agents history
 * and determines how it moved after that sequence of moves
 * previously seen.
 * 
 * @author jberg
 *
 */

public class PrevSequenceIndividualOracle extends AbstractPrevSequenceOracle {

	public PrevSequenceIndividualOracle(Game g, long seed, String options) {
		super(g,seed,options);
	}
	
	public int[] getOppDist(int opp) {
		int[] arr = new int[12];
		ArrayList<Integer> moves;
		if(opp == 0) {
			moves = player1Moves;
		}
		else if (opp == 1) {
			moves = player2Moves;
		}
		else {
			moves = player3Moves;
		}
		if(_numLastMoves == 0 || moves.size() <= _numLastMoves) {
			for (Integer move : moves) {
				arr[move] = arr[move] + 1;
			}
		}
		else {
			List<Integer> prevMoves = moves.subList(moves.size() - _numLastMoves, moves.size());
			for(int i = 0; i < moves.size() - _numLastMoves; i++) {
				List<Integer> subMoves = moves.subList(i, i+_numLastMoves);
				if (arrListEquals(prevMoves, subMoves)) {
					arr[moves.get(i+_numLastMoves)] = arr[moves.get(i+_numLastMoves)] + 1;
				}
			}
		}
		return arr;
	}
	
	public String toString() {
		return "PrevSequenceIndividualOracle(n = " + _numLastMoves + ")";
	}
	
	public PrevSequenceIndividualOracle getCopy() {
		return new PrevSequenceIndividualOracle(_g,_seed,_options);
	}
	
	@Override
	public PrevSequenceIndividualOracle getCopy(Game g, long seed, String options) {
		return new PrevSequenceIndividualOracle(g,seed,options);
	}
	
}