package main;

import game.Game;
import java.lang.reflect.InvocationTargetException;
import java.util.Random;
import util.Array;
import lemonade.ScaledLemonadeUtility;


/**
 * A main class for running the Lemonade tournament.
 * Every triple of players is played.
 * No variance reduction tricks are used, nor is variance calculated.
 * Usage: <SEED> <PLAYER1> <OPTION1> <PLAYER2> <OPTION2> <PLAYER3> <OPTION3>
 * @author maz
 *
 */
public class RunLemonadeTournament {
  
  /**
   * Run a tournament with a symmetric game.
   * Every triple of players is played.
   * No variance reduction tricks are used, nor is variance calculated.
   */
  public static void runTournament(Game g, String[] strategies, String[] options, Random seedGenerator,int iterations,int repeats) throws IllegalArgumentException, InstantiationException, IllegalAccessException, InvocationTargetException, ClassNotFoundException, NoSuchMethodException{
    
    
    for(int i=0;i<strategies.length;i++){
      for(int j=i+1;j<strategies.length;j++){
        for(int k=j+1;k<strategies.length;k++){
          String[] threeStrategies = {strategies[i],strategies[j],strategies[k]};
          String[] threeOptions = {options[i],options[j],options[k]};
          
          System.out.println(Array.arrayToNakedString(threeStrategies));
          long[] squares = new long[3];
          int[] totals = RunLemonadeGame.manyRuns(g, threeStrategies, threeOptions, seedGenerator,iterations,repeats,squares);
          System.out.println(Array.arrayToNakedString(totals));
          System.out.println(Array.arrayToNakedString(squares));
        }
      }
    }
  }

  public static void showUsage(){
    System.err.println("Usage:RunLemonadeTournament [-seed <SEED>|-numActions <ACTIONS>|-numRounds<ROUNDS> |-numRepeats <REPEATS>]* -players [<CLASS> <OPTION>]*");
    System.exit(0);
  }

  /**
   * Run a tournament with a symmetric game.
   * Every triple of players is played.
   * No variance reduction tricks are used, nor is variance calculated.
   */
  public static void main(String[] args) throws ClassNotFoundException, SecurityException, NoSuchMethodException, IllegalArgumentException, InstantiationException, IllegalAccessException, InvocationTargetException{
    
    if (args.length<4){
      showUsage();
    }
    
    long seed=0;
    int numActions=12;
    int iterations = 100;
    int repeats = 1000;
    int playerArgIndex=args.length;
    for(int i=0;i<args.length;i+=2){
      String arg = args[i];
      String fieldStr = args[i+1];
      if (arg.equals("-seed")){
        long field = Long.parseLong(fieldStr);
        seed = field;
      } else if (arg.equals("-actions")){
        long field = Long.parseLong(fieldStr);
        if (100L<field){
          System.err.println("Number of actions needs to be less than 100");
          showUsage();
        }
        numActions = (int)field;
      } else if (arg.equals("-rounds")){
        long field = Long.parseLong(fieldStr);
        if ((long)(Integer.MAX_VALUE)<field){
          System.err.println("Number of rounds needs to be less than 2^31-1");
          showUsage();
        }
        iterations = (int)field;
      } else if (arg.equals("-repeats")){
        long field = Long.parseLong(fieldStr);
        if ((long)(Integer.MAX_VALUE)<field){
          System.err.println("Number of repeats needs to be less than 2^31-1");
          showUsage();
        }
        repeats = (int)field;
      } else if (arg.equals("-players")){
        playerArgIndex=i+1;
        break;
      } else {
        System.err.println("Unknown option:"+arg);
        showUsage();
        
      }
    }

    Random seedGenerator = new Random(seed);
    ScaledLemonadeUtility utility = new ScaledLemonadeUtility(numActions);
    Game g = new Game(new int[]{numActions,numActions,numActions},utility);

    String[] strategies = new String[(args.length-playerArgIndex)/2];
    String[] options = new String[strategies.length];
    int playerIndex = 0;
    for(int i=playerArgIndex;i<args.length;i+=2){
      strategies[playerIndex]=args[i];
      options[playerIndex]=args[i+1];
      playerIndex++;
    }
    runTournament(g, strategies, options, seedGenerator,iterations,repeats);
  }
}
